package java.util.concurrent;

import java.util.concurrent.locks.LockSupport;
import java.util.function.Supplier;
import java.util.function.BiFunction;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;
import sun.misc.Unsafe;

public class CompletableFuture<T> implements Future<T>, CompletionStage<T>
{
    volatile Object result;
    volatile CompletableFuture.Completion stack;
    static final CompletableFuture.AltResult NIL;
    private static final boolean useCommonPool;
    private static final Executor asyncPool;
    static final int SYNC = 0;
    static final int ASYNC = 1;
    static final int NESTED = -1;
    private static final int SPINS;
    private static final Unsafe UNSAFE;
    private static final long RESULT;
    private static final long STACK;
    private static final long NEXT;
    
    final boolean internalComplete(Object x) {
        return CompletableFuture.UNSAFE.compareAndSwapObject(this, CompletableFuture.RESULT, null, x);
    }
    
    final boolean casStack(CompletableFuture.Completion expected, CompletableFuture.Completion x) {
        return CompletableFuture.UNSAFE.compareAndSwapObject(this, CompletableFuture.STACK, expected, x);
    }
    
    final boolean tryPushStack(CompletableFuture.Completion x) {
        CompletableFuture.Completion stack = this.stack;
        lazySetNext(x, stack);
        return CompletableFuture.UNSAFE.compareAndSwapObject(this, CompletableFuture.STACK, stack, x);
    }
    
    final void pushStack(CompletableFuture.Completion completion) {
        while (!this.tryPushStack(completion)) {}
    }
    
    final boolean completeNull() {
        return CompletableFuture.UNSAFE.compareAndSwapObject(this, CompletableFuture.RESULT, null, CompletableFuture.NIL);
    }
    
    final Object encodeValue(T t) {
        return (t == null) ? CompletableFuture.NIL : t;
    }
    
    final boolean completeValue(T t) {
        return CompletableFuture.UNSAFE.compareAndSwapObject(this, CompletableFuture.RESULT, null, (t == null) ? CompletableFuture.NIL : t);
    }
    
    static CompletableFuture.AltResult encodeThrowable(Throwable t) {
        return new CompletableFuture.AltResult((t instanceof CompletionException) ? t : new CompletionException(t));
    }
    
    final boolean completeThrowable(Throwable t) {
        return CompletableFuture.UNSAFE.compareAndSwapObject(this, CompletableFuture.RESULT, null, encodeThrowable(t));
    }
    
    static Object encodeThrowable(Throwable t, Object o) {
        if (!(t instanceof CompletionException)) {
            t = new CompletionException(t);
        }
        else if (o instanceof CompletableFuture.AltResult && t == ((CompletableFuture.AltResult)o).ex) {
            return o;
        }
        return new AltResult(t);
    }
    
    final boolean completeThrowable(Throwable t, Object o) {
        return CompletableFuture.UNSAFE.compareAndSwapObject(this, CompletableFuture.RESULT, null, encodeThrowable(t, o));
    }
    
    Object encodeOutcome(T t, Throwable t2) {
        return (t2 == null) ? ((t == null) ? CompletableFuture.NIL : t) : encodeThrowable(t2);
    }
    
    static Object encodeRelay(Object o) {
        Throwable ex;
        return (o instanceof CompletableFuture.AltResult && (ex = ((CompletableFuture.AltResult)o).ex) != null && !(ex instanceof CompletionException)) ? new CompletableFuture.AltResult(new CompletionException(ex)) : o;
    }
    
    final boolean completeRelay(Object o) {
        return CompletableFuture.UNSAFE.compareAndSwapObject(this, CompletableFuture.RESULT, null, encodeRelay(o));
    }
    
    private static <T> T reportGet(Object o) throws InterruptedException, ExecutionException {
        if (o == null) {
            throw new InterruptedException();
        }
        if ((o instanceof CompletableFuture.AltResult)) {
            Throwable ex;
            if ((ex = ((CompletableFuture.AltResult)o).ex) == null) {
                return null;
            }
            if ((ex instanceof CancellationException)) {
                throw (CancellationException)ex;
            }
            Throwable cause;
            if (ex instanceof CompletionException && (cause = ex.getCause()) != null) {
                ex = cause;
            }
            throw new ExecutionException(ex);
        }
        else {
            return o;
        }
    }
    
    private static <T> T reportJoin(Object o) {
        if ((o instanceof CompletableFuture.AltResult)) {
            Throwable ex;
            if ((ex = ((CompletableFuture.AltResult)o).ex) == null) {
                return null;
            }
            if ((ex instanceof CancellationException)) {
                throw (CancellationException)ex;
            }
            if ((ex instanceof CompletionException)) {
                throw (CompletionException)ex;
            }
            throw new CompletionException(ex);
        }
        else {
            return o;
        }
    }
    
    static Executor screenExecutor(Executor executor) {
        if (!CompletableFuture.useCommonPool && executor == ForkJoinPool.commonPool()) {
            return CompletableFuture.asyncPool;
        }
        if (executor == null) {
            throw new NullPointerException();
        }
        return executor;
    }
    
    static void lazySetNext(CompletableFuture.Completion o, CompletableFuture.Completion x) {
        CompletableFuture.UNSAFE.putOrderedObject(o, CompletableFuture.NEXT, x);
    }
    
    final void postComplete() {
        CompletableFuture.Completion completion;
        while ((completion = this.stack) != null || (this != this && (completion = (this = this).stack) != null)) {
            CompletableFuture.Completion next;
            if (this.casStack(completion, next = completion.next)) {
                if (next != null) {
                    if (this != this) {
                        this.pushStack(completion);
                        continue;
                    }
                    completion.next = null;
                }
                CompletableFuture<Object> tryFire;
                this = (((tryFire = completion.tryFire(-1)) == null) ? this : tryFire);
            }
            else {
                continue;
            }
        }
    }
    
    final void cleanStack() {
        CompletableFuture.Completion completion = null;
        CompletableFuture.Completion completion2 = this.stack;
        while (completion2 != null) {
            CompletableFuture.Completion next = completion2.next;
            if (completion2.isLive()) {
                completion = completion2;
                completion2 = next;
            }
            else if (completion == null) {
                this.casStack(completion2, next);
                completion2 = this.stack;
            }
            else {
                completion.next = next;
                if (completion.isLive()) {
                    completion2 = next;
                }
                else {
                    completion = null;
                    completion2 = this.stack;
                }
            }
        }
    }
    
    final void push(CompletableFuture.UniCompletion<?, ?> uniCompletion) {
        if (uniCompletion != null) {
            while (this.result == null && !this.tryPushStack(uniCompletion)) {
                lazySetNext(uniCompletion, null);
            }
        }
    }
    
    final CompletableFuture<T> postFire(CompletableFuture<?> completableFuture, int n) {
        if (completableFuture != null && completableFuture.stack != null) {
            if (n < 0 || completableFuture.result == null) {
                completableFuture.cleanStack();
            }
            else {
                completableFuture.postComplete();
            }
        }
        if (this.result != null && this.stack != null) {
            if (n < 0) {
                return this;
            }
            this.postComplete();
        }
        return null;
    }
    
    final <S> boolean uniApply(CompletableFuture<S> completableFuture, Function<? super S, ? extends T> function, CompletableFuture.UniApply<S, T> uniApply) {
        Object result;
        if (completableFuture == null || (result = completableFuture.result) == null || function == null) {
            return false;
        }
        if (this.result == null) {
            if ((result instanceof CompletableFuture.AltResult)) {
                Throwable ex;
                if ((ex = ((CompletableFuture.AltResult)result).ex) != null) {
                    this.completeThrowable(ex, result);
                    return true;
                }
                result = null;
            }
            try {
                if (uniApply != null && !uniApply.claim()) {
                    return false;
                }
                this.completeValue(function.apply(result));
            }
            catch (Throwable t) {
                this.completeThrowable(t);
            }
        }
        return true;
    }
    
    private <V> CompletableFuture<V> uniApplyStage(Executor executor, Function<? super T, ? extends V> function) {
        if (function == null) {
            throw new NullPointerException();
        }
        CompletableFuture<Object> completableFuture = new CompletableFuture();
        if (executor != null || !completableFuture.uniApply(this, function, null)) {
            CompletableFuture.UniApply<Object, Object> uniApply = new CompletableFuture.UniApply(executor, completableFuture, this, function);
            this.push(uniApply);
            uniApply.tryFire(0);
            return completableFuture;
        }
        return completableFuture;
    }
    
    final <S> boolean uniAccept(CompletableFuture<S> completableFuture, Consumer<? super S> consumer, CompletableFuture.UniAccept<S> uniAccept) {
        Object result;
        if (completableFuture == null || (result = completableFuture.result) == null || consumer == null) {
            return false;
        }
        if (this.result == null) {
            if ((result instanceof CompletableFuture.AltResult)) {
                Throwable ex;
                if ((ex = ((CompletableFuture.AltResult)result).ex) != null) {
                    this.completeThrowable(ex, result);
                    return true;
                }
                result = null;
            }
            try {
                if (uniAccept != null && !uniAccept.claim()) {
                    return false;
                }
                consumer.accept(result);
                this.completeNull();
            }
            catch (Throwable t) {
                this.completeThrowable(t);
            }
        }
        return true;
    }
    
    private CompletableFuture<Void> uniAcceptStage(Executor executor, Consumer<? super T> consumer) {
        if (consumer == null) {
            throw new NullPointerException();
        }
        CompletableFuture<Object> completableFuture = new CompletableFuture();
        if (executor != null || !completableFuture.uniAccept(this, consumer, null)) {
            CompletableFuture.UniAccept<Object> uniAccept = new CompletableFuture.UniAccept(executor, completableFuture, this, consumer);
            this.push(uniAccept);
            uniAccept.tryFire(0);
            return completableFuture;
        }
        return completableFuture;
    }
    
    final boolean uniRun(CompletableFuture<?> completableFuture, Runnable runnable, CompletableFuture.UniRun<?> uniRun) {
        Object result;
        if (completableFuture == null || (result = completableFuture.result) == null || runnable == null) {
            return false;
        }
        if (this.result == null) {
            Throwable ex;
            if (result instanceof CompletableFuture.AltResult && (ex = ((CompletableFuture.AltResult)result).ex) != null) {
                this.completeThrowable(ex, result);
            }
            else {
                try {
                    if (uniRun != null && !uniRun.claim()) {
                        return false;
                    }
                    runnable.run();
                    this.completeNull();
                }
                catch (Throwable t) {
                    this.completeThrowable(t);
                }
            }
        }
        return true;
    }
    
    private CompletableFuture<Void> uniRunStage(Executor executor, Runnable runnable) {
        if (runnable == null) {
            throw new NullPointerException();
        }
        CompletableFuture<Object> completableFuture = new CompletableFuture();
        if (executor != null || !completableFuture.uniRun(this, runnable, null)) {
            CompletableFuture.UniRun<Object> uniRun = new CompletableFuture.UniRun(executor, completableFuture, this, runnable);
            this.push(uniRun);
            uniRun.tryFire(0);
            return completableFuture;
        }
        return completableFuture;
    }
    
    final boolean uniWhenComplete(CompletableFuture<T> completableFuture, BiConsumer<? super T, ? super Throwable> biConsumer, CompletableFuture.UniWhenComplete<T> uniWhenComplete) {
        Throwable ex = null;
        Object result;
        if (completableFuture == null || (result = completableFuture.result) == null || biConsumer == null) {
            return false;
        }
        if (this.result == null) {
            try {
                if (uniWhenComplete != null && !uniWhenComplete.claim()) {
                    return false;
                }
                Object o;
                if ((result instanceof CompletableFuture.AltResult)) {
                    ex = ((CompletableFuture.AltResult)result).ex;
                    o = null;
                }
                else {
                    result = (o = result);
                }
                biConsumer.accept(o, ex);
                if (ex == null) {
                    this.internalComplete(result);
                    return true;
                }
            }
            catch (Throwable t) {
                if (ex == null) {
                    ex = t;
                }
            }
            this.completeThrowable(ex, result);
        }
        return true;
    }
    
    private CompletableFuture<T> uniWhenCompleteStage(Executor executor, BiConsumer<? super T, ? super Throwable> biConsumer) {
        if (biConsumer == null) {
            throw new NullPointerException();
        }
        CompletableFuture<Object> completableFuture = new CompletableFuture();
        if (executor != null || !completableFuture.uniWhenComplete(this, biConsumer, null)) {
            CompletableFuture.UniWhenComplete<Object> uniWhenComplete = new CompletableFuture.UniWhenComplete(executor, completableFuture, this, biConsumer);
            this.push(uniWhenComplete);
            uniWhenComplete.tryFire(0);
            return completableFuture;
        }
        return completableFuture;
    }
    
    final <S> boolean uniHandle(CompletableFuture<S> completableFuture, BiFunction<? super S, Throwable, ? extends T> biFunction, CompletableFuture.UniHandle<S, T> uniHandle) {
        Object result;
        if (completableFuture == null || (result = completableFuture.result) == null || biFunction == null) {
            return false;
        }
        if (this.result == null) {
            try {
                if (uniHandle != null && !uniHandle.claim()) {
                    return false;
                }
                Throwable ex;
                Object o;
                if ((result instanceof CompletableFuture.AltResult)) {
                    ex = ((CompletableFuture.AltResult)result).ex;
                    o = null;
                }
                else {
                    ex = null;
                    result = (o = result);
                }
                this.completeValue(biFunction.apply(o, ex));
            }
            catch (Throwable t) {
                this.completeThrowable(t);
            }
        }
        return true;
    }
    
    private <V> CompletableFuture<V> uniHandleStage(Executor executor, BiFunction<? super T, Throwable, ? extends V> biFunction) {
        if (biFunction == null) {
            throw new NullPointerException();
        }
        CompletableFuture<Object> completableFuture = new CompletableFuture();
        if (executor != null || !completableFuture.uniHandle(this, biFunction, null)) {
            CompletableFuture.UniHandle<Object, Object> uniHandle = new CompletableFuture.UniHandle(executor, completableFuture, this, biFunction);
            this.push(uniHandle);
            uniHandle.tryFire(0);
            return completableFuture;
        }
        return completableFuture;
    }
    
    final boolean uniExceptionally(CompletableFuture<T> completableFuture, Function<? super Throwable, ? extends T> function, CompletableFuture.UniExceptionally<T> uniExceptionally) {
        Object result;
        if (completableFuture == null || (result = completableFuture.result) == null || function == null) {
            return false;
        }
        if (this.result == null) {
            try {
                Throwable ex;
                if (result instanceof CompletableFuture.AltResult && (ex = ((CompletableFuture.AltResult)result).ex) != null) {
                    if (uniExceptionally != null && !uniExceptionally.claim()) {
                        return false;
                    }
                    this.completeValue(function.apply(ex));
                }
                else {
                    this.internalComplete(result);
                }
            }
            catch (Throwable t) {
                this.completeThrowable(t);
            }
        }
        return true;
    }
    
    private CompletableFuture<T> uniExceptionallyStage(Function<Throwable, ? extends T> function) {
        if (function == null) {
            throw new NullPointerException();
        }
        CompletableFuture<Object> completableFuture = new CompletableFuture();
        if (!completableFuture.uniExceptionally(this, function, null)) {
            CompletableFuture.UniExceptionally<Object> uniExceptionally = new CompletableFuture.UniExceptionally(completableFuture, this, function);
            this.push(uniExceptionally);
            uniExceptionally.tryFire(0);
            return completableFuture;
        }
        return completableFuture;
    }
    
    final boolean uniRelay(CompletableFuture<T> completableFuture) {
        Object result;
        if (completableFuture == null || (result = completableFuture.result) == null) {
            return false;
        }
        if (this.result == null) {
            this.completeRelay(result);
        }
        return true;
    }
    
    final <S> boolean uniCompose(CompletableFuture<S> completableFuture, Function<? super S, ? extends CompletionStage<T>> function, CompletableFuture.UniCompose<S, T> uniCompose) {
        Object result;
        if (completableFuture == null || (result = completableFuture.result) == null || function == null) {
            return false;
        }
        if (this.result == null) {
            if ((result instanceof CompletableFuture.AltResult)) {
                Throwable ex;
                if ((ex = ((CompletableFuture.AltResult)result).ex) != null) {
                    this.completeThrowable(ex, result);
                    return true;
                }
                result = null;
            }
            try {
                if (uniCompose != null && !uniCompose.claim()) {
                    return false;
                }
                CompletableFuture<T> completableFuture2 = ((CompletionStage)function.apply(result)).toCompletableFuture();
                if (completableFuture2.result == null || !this.uniRelay(completableFuture2)) {
                    CompletableFuture.UniRelay o = new CompletableFuture.UniRelay(this, completableFuture2);
                    completableFuture2.push(o);
                    o.tryFire(0);
                    if (this.result == null) {
                        return false;
                    }
                }
            }
            catch (Throwable t) {
                this.completeThrowable(t);
            }
        }
        return true;
    }
    
    private <V> CompletableFuture<V> uniComposeStage(Executor executor, Function<? super T, ? extends CompletionStage<V>> function) {
        if (function == null) {
            throw new NullPointerException();
        }
        Object result;
        if (executor == null && (result = this.result) != null) {
            if ((result instanceof CompletableFuture.AltResult)) {
                Throwable ex;
                if ((ex = ((CompletableFuture.AltResult)result).ex) != null) {
                    return new CompletableFuture(encodeThrowable(ex, result));
                }
                result = null;
            }
            try {
                CompletableFuture<Object> completableFuture = ((CompletionStage)function.apply(result)).toCompletableFuture();
                Object result2 = completableFuture.result;
                if (result2 != null) {
                    return new CompletableFuture(encodeRelay(result2));
                }
                CompletableFuture o3 = new CompletableFuture();
                CompletableFuture.UniRelay o4 = new CompletableFuture.UniRelay(o3, completableFuture);
                completableFuture.push(o4);
                o4.tryFire(0);
                return o3;
            }
            catch (Throwable t) {
                return new CompletableFuture(encodeThrowable(t));
            }
        }
        else {
            CompletableFuture o = new CompletableFuture();
            CompletableFuture.UniCompose o2 = new CompletableFuture.UniCompose(executor, o, this, function);
            this.push(o2);
            o2.tryFire(0);
            return o;
        }
    }
    
    final void bipush(CompletableFuture<?> completableFuture, CompletableFuture.BiCompletion<?, ?, ?> biCompletion) {
        if (biCompletion != null) {
            Object result;
            while ((result = this.result) == null && !this.tryPushStack(biCompletion)) {
                lazySetNext(biCompletion, null);
            }
            if (completableFuture != null && completableFuture != this && completableFuture.result == null) {
                CompletableFuture.Completion completion = (result != null) ? biCompletion : new CoCompletion(biCompletion);
                while (completableFuture.result == null && !completableFuture.tryPushStack(completion)) {
                    lazySetNext(completion, null);
                }
            }
        }
    }
    
    final CompletableFuture<T> postFire(CompletableFuture<?> completableFuture, CompletableFuture<?> completableFuture2, int n) {
        if (completableFuture2 != null && completableFuture2.stack != null) {
            if (n < 0 || completableFuture2.result == null) {
                completableFuture2.cleanStack();
            }
            else {
                completableFuture2.postComplete();
            }
        }
        return this.postFire(completableFuture, n);
    }
    
    final <R, S> boolean biApply(CompletableFuture<R> completableFuture, CompletableFuture<S> completableFuture2, BiFunction<? super R, ? super S, ? extends T> biFunction, CompletableFuture.BiApply<R, S, T> biApply) {
        Object result;
        Object result2;
        if (completableFuture == null || (result = completableFuture.result) == null || completableFuture2 == null || (result2 = completableFuture2.result) == null || biFunction == null) {
            return false;
        }
        if (this.result == null) {
            if ((result instanceof CompletableFuture.AltResult)) {
                Throwable ex;
                if ((ex = ((CompletableFuture.AltResult)result).ex) != null) {
                    this.completeThrowable(ex, result);
                    return true;
                }
                result = null;
            }
            if ((result2 instanceof CompletableFuture.AltResult)) {
                Throwable ex2;
                if ((ex2 = ((CompletableFuture.AltResult)result2).ex) != null) {
                    this.completeThrowable(ex2, result2);
                    return true;
                }
                result2 = null;
            }
            try {
                if (biApply != null && !biApply.claim()) {
                    return false;
                }
                this.completeValue(biFunction.apply(result, result2));
            }
            catch (Throwable t) {
                this.completeThrowable(t);
            }
        }
        return true;
    }
    
    private <U, V> CompletableFuture<V> biApplyStage(Executor executor, CompletionStage<U> completionStage, BiFunction<? super T, ? super U, ? extends V> biFunction) {
        CompletableFuture<Object> completableFuture;
        if (biFunction == null || (completableFuture = completionStage.toCompletableFuture()) == null) {
            throw new NullPointerException();
        }
        CompletableFuture<Object> completableFuture2 = new CompletableFuture();
        if (executor != null || !completableFuture2.biApply(this, completableFuture, biFunction, null)) {
            CompletableFuture.BiApply<Object, Object, Object> biApply = new CompletableFuture.BiApply(executor, completableFuture2, this, completableFuture, biFunction);
            this.bipush(completableFuture, biApply);
            biApply.tryFire(0);
            return completableFuture2;
        }
        return completableFuture2;
    }
    
    final <R, S> boolean biAccept(CompletableFuture<R> completableFuture, CompletableFuture<S> completableFuture2, BiConsumer<? super R, ? super S> biConsumer, CompletableFuture.BiAccept<R, S> biAccept) {
        Object result;
        Object result2;
        if (completableFuture == null || (result = completableFuture.result) == null || completableFuture2 == null || (result2 = completableFuture2.result) == null || biConsumer == null) {
            return false;
        }
        if (this.result == null) {
            if ((result instanceof CompletableFuture.AltResult)) {
                Throwable ex;
                if ((ex = ((CompletableFuture.AltResult)result).ex) != null) {
                    this.completeThrowable(ex, result);
                    return true;
                }
                result = null;
            }
            if ((result2 instanceof CompletableFuture.AltResult)) {
                Throwable ex2;
                if ((ex2 = ((CompletableFuture.AltResult)result2).ex) != null) {
                    this.completeThrowable(ex2, result2);
                    return true;
                }
                result2 = null;
            }
            try {
                if (biAccept != null && !biAccept.claim()) {
                    return false;
                }
                biConsumer.accept(result, result2);
                this.completeNull();
            }
            catch (Throwable t) {
                this.completeThrowable(t);
            }
        }
        return true;
    }
    
    private <U> CompletableFuture<Void> biAcceptStage(Executor executor, CompletionStage<U> completionStage, BiConsumer<? super T, ? super U> biConsumer) {
        CompletableFuture<Object> completableFuture;
        if (biConsumer == null || (completableFuture = completionStage.toCompletableFuture()) == null) {
            throw new NullPointerException();
        }
        CompletableFuture<Object> completableFuture2 = new CompletableFuture();
        if (executor != null || !completableFuture2.biAccept(this, completableFuture, biConsumer, null)) {
            CompletableFuture.BiAccept<Object, Object> biAccept = new CompletableFuture.BiAccept(executor, completableFuture2, this, completableFuture, biConsumer);
            this.bipush(completableFuture, biAccept);
            biAccept.tryFire(0);
            return completableFuture2;
        }
        return completableFuture2;
    }
    
    final boolean biRun(CompletableFuture<?> completableFuture, CompletableFuture<?> completableFuture2, Runnable runnable, CompletableFuture.BiRun<?, ?> biRun) {
        Object result;
        Object result2;
        if (completableFuture == null || (result = completableFuture.result) == null || completableFuture2 == null || (result2 = completableFuture2.result) == null || runnable == null) {
            return false;
        }
        if (this.result == null) {
            Throwable ex;
            if (result instanceof CompletableFuture.AltResult && (ex = ((CompletableFuture.AltResult)result).ex) != null) {
                this.completeThrowable(ex, result);
            }
            else {
                Throwable ex2;
                if (result2 instanceof CompletableFuture.AltResult && (ex2 = ((CompletableFuture.AltResult)result2).ex) != null) {
                    this.completeThrowable(ex2, result2);
                }
                else {
                    try {
                        if (biRun != null && !biRun.claim()) {
                            return false;
                        }
                        runnable.run();
                        this.completeNull();
                    }
                    catch (Throwable t) {
                        this.completeThrowable(t);
                    }
                }
            }
        }
        return true;
    }
    
    private CompletableFuture<Void> biRunStage(Executor executor, CompletionStage<?> completionStage, Runnable runnable) {
        CompletableFuture<Object> completableFuture;
        if (runnable == null || (completableFuture = completionStage.toCompletableFuture()) == null) {
            throw new NullPointerException();
        }
        CompletableFuture<Object> completableFuture2 = new CompletableFuture();
        if (executor != null || !completableFuture2.biRun(this, completableFuture, runnable, null)) {
            CompletableFuture.BiRun<Object, Object> biRun = new CompletableFuture.BiRun(executor, completableFuture2, this, completableFuture, runnable);
            this.bipush(completableFuture, biRun);
            biRun.tryFire(0);
            return completableFuture2;
        }
        return completableFuture2;
    }
    
    boolean biRelay(CompletableFuture<?> completableFuture, CompletableFuture<?> completableFuture2) {
        Object result;
        Object result2;
        if (completableFuture == null || (result = completableFuture.result) == null || completableFuture2 == null || (result2 = completableFuture2.result) == null) {
            return false;
        }
        if (this.result == null) {
            Throwable ex;
            if (result instanceof CompletableFuture.AltResult && (ex = ((CompletableFuture.AltResult)result).ex) != null) {
                this.completeThrowable(ex, result);
            }
            else {
                Throwable ex2;
                if (result2 instanceof CompletableFuture.AltResult && (ex2 = ((CompletableFuture.AltResult)result2).ex) != null) {
                    this.completeThrowable(ex2, result2);
                }
                else {
                    this.completeNull();
                }
            }
        }
        return true;
    }
    
    static CompletableFuture<Void> andTree(CompletableFuture<?>[] array, int n, int n2) {
        CompletableFuture<Object> completableFuture = new CompletableFuture();
        if (n > n2) {
            completableFuture.result = CompletableFuture.NIL;
        }
        else {
            int n3 = n + n2 >>> 1;
            CompletableFuture<?> completableFuture2;
            if ((completableFuture2 = ((n == n3) ? array[n] : andTree(array, n, n3))) != null) {
                CompletableFuture<?> completableFuture4;
                CompletableFuture<?> completableFuture3;
                if (n == n2) {
                    completableFuture3 = completableFuture2;
                    completableFuture4 = completableFuture2;
                }
                else {
                    completableFuture3 = (completableFuture4 = ((n2 == n3 + 1) ? array[n2] : andTree(array, n3 + 1, n2)));
                }
                CompletableFuture completableFuture5 = completableFuture4;
                if (completableFuture3 != null) {
                    if (!completableFuture.biRelay(completableFuture2, completableFuture5)) {
                        CompletableFuture.BiRelay o = new CompletableFuture.BiRelay(completableFuture, completableFuture2, completableFuture5);
                        completableFuture2.bipush(completableFuture5, o);
                        o.tryFire(0);
                        return completableFuture;
                    }
                    return completableFuture;
                }
            }
            throw new NullPointerException();
        }
        return completableFuture;
    }
    
    final void orpush(CompletableFuture<?> completableFuture, CompletableFuture.BiCompletion<?, ?, ?> biCompletion) {
        if (biCompletion != null) {
            while ((completableFuture == null || completableFuture.result == null) && this.result == null) {
                if (this.tryPushStack(biCompletion)) {
                    if (completableFuture != null && completableFuture != this && completableFuture.result == null) {
                        CompletableFuture.CoCompletion coCompletion = new CoCompletion(biCompletion);
                        while (this.result == null && completableFuture.result == null && !completableFuture.tryPushStack(coCompletion)) {
                            lazySetNext(coCompletion, null);
                        }
                        break;
                    }
                    break;
                }
                else {
                    lazySetNext(biCompletion, null);
                }
            }
        }
    }
    
    final <R, S extends R> boolean orApply(CompletableFuture<R> completableFuture, CompletableFuture<S> completableFuture2, Function<? super R, ? extends T> function, CompletableFuture.OrApply<R, S, T> orApply) {
        Object o;
        if (completableFuture == null || completableFuture2 == null || ((o = completableFuture.result) == null && (o = completableFuture2.result) == null) || function == null) {
            return false;
        }
        if (this.result == null) {
            try {
                if (orApply != null && !orApply.claim()) {
                    return false;
                }
                if ((o instanceof CompletableFuture.AltResult)) {
                    Throwable ex;
                    if ((ex = ((CompletableFuture.AltResult)o).ex) != null) {
                        this.completeThrowable(ex, o);
                        return true;
                    }
                    o = null;
                }
                this.completeValue(function.apply(o));
            }
            catch (Throwable t) {
                this.completeThrowable(t);
            }
        }
        return true;
    }
    
    private <U extends T, V> CompletableFuture<V> orApplyStage(Executor executor, CompletionStage<U> completionStage, Function<? super T, ? extends V> function) {
        CompletableFuture<Object> completableFuture;
        if (function == null || (completableFuture = completionStage.toCompletableFuture()) == null) {
            throw new NullPointerException();
        }
        CompletableFuture<Object> completableFuture2 = new CompletableFuture();
        if (executor != null || !completableFuture2.orApply(this, completableFuture, function, null)) {
            CompletableFuture.OrApply<Object, Object, Object> orApply = new CompletableFuture.OrApply(executor, completableFuture2, this, completableFuture, function);
            this.orpush(completableFuture, orApply);
            orApply.tryFire(0);
            return completableFuture2;
        }
        return completableFuture2;
    }
    
    final <R, S extends R> boolean orAccept(CompletableFuture<R> completableFuture, CompletableFuture<S> completableFuture2, Consumer<? super R> consumer, CompletableFuture.OrAccept<R, S> orAccept) {
        Object o;
        if (completableFuture == null || completableFuture2 == null || ((o = completableFuture.result) == null && (o = completableFuture2.result) == null) || consumer == null) {
            return false;
        }
        if (this.result == null) {
            try {
                if (orAccept != null && !orAccept.claim()) {
                    return false;
                }
                if ((o instanceof CompletableFuture.AltResult)) {
                    Throwable ex;
                    if ((ex = ((CompletableFuture.AltResult)o).ex) != null) {
                        this.completeThrowable(ex, o);
                        return true;
                    }
                    o = null;
                }
                consumer.accept(o);
                this.completeNull();
            }
            catch (Throwable t) {
                this.completeThrowable(t);
            }
        }
        return true;
    }
    
    private <U extends T> CompletableFuture<Void> orAcceptStage(Executor executor, CompletionStage<U> completionStage, Consumer<? super T> consumer) {
        CompletableFuture<Object> completableFuture;
        if (consumer == null || (completableFuture = completionStage.toCompletableFuture()) == null) {
            throw new NullPointerException();
        }
        CompletableFuture<Object> completableFuture2 = new CompletableFuture();
        if (executor != null || !completableFuture2.orAccept(this, completableFuture, consumer, null)) {
            CompletableFuture.OrAccept<Object, Object> orAccept = new CompletableFuture.OrAccept(executor, completableFuture2, this, completableFuture, consumer);
            this.orpush(completableFuture, orAccept);
            orAccept.tryFire(0);
            return completableFuture2;
        }
        return completableFuture2;
    }
    
    final boolean orRun(CompletableFuture<?> completableFuture, CompletableFuture<?> completableFuture2, Runnable runnable, CompletableFuture.OrRun<?, ?> orRun) {
        Object o;
        if (completableFuture == null || completableFuture2 == null || ((o = completableFuture.result) == null && (o = completableFuture2.result) == null) || runnable == null) {
            return false;
        }
        if (this.result == null) {
            try {
                if (orRun != null && !orRun.claim()) {
                    return false;
                }
                Throwable ex;
                if (o instanceof CompletableFuture.AltResult && (ex = ((CompletableFuture.AltResult)o).ex) != null) {
                    this.completeThrowable(ex, o);
                }
                else {
                    runnable.run();
                    this.completeNull();
                }
            }
            catch (Throwable t) {
                this.completeThrowable(t);
            }
        }
        return true;
    }
    
    private CompletableFuture<Void> orRunStage(Executor executor, CompletionStage<?> completionStage, Runnable runnable) {
        CompletableFuture<Object> completableFuture;
        if (runnable == null || (completableFuture = completionStage.toCompletableFuture()) == null) {
            throw new NullPointerException();
        }
        CompletableFuture<Object> completableFuture2 = new CompletableFuture();
        if (executor != null || !completableFuture2.orRun(this, completableFuture, runnable, null)) {
            CompletableFuture.OrRun<Object, Object> orRun = new CompletableFuture.OrRun(executor, completableFuture2, this, completableFuture, runnable);
            this.orpush(completableFuture, orRun);
            orRun.tryFire(0);
            return completableFuture2;
        }
        return completableFuture2;
    }
    
    final boolean orRelay(CompletableFuture<?> completableFuture, CompletableFuture<?> completableFuture2) {
        Object o;
        if (completableFuture == null || completableFuture2 == null || ((o = completableFuture.result) == null && (o = completableFuture2.result) == null)) {
            return false;
        }
        if (this.result == null) {
            this.completeRelay(o);
        }
        return true;
    }
    
    static CompletableFuture<Object> orTree(CompletableFuture<?>[] array, int n, int n2) {
        CompletableFuture<Object> completableFuture = new CompletableFuture();
        if (n <= n2) {
            int n3 = n + n2 >>> 1;
            CompletableFuture<?> completableFuture2;
            if ((completableFuture2 = ((n == n3) ? array[n] : orTree(array, n, n3))) != null) {
                CompletableFuture<?> completableFuture4;
                CompletableFuture<?> completableFuture3;
                if (n == n2) {
                    completableFuture3 = completableFuture2;
                    completableFuture4 = completableFuture2;
                }
                else {
                    completableFuture3 = (completableFuture4 = ((n2 == n3 + 1) ? array[n2] : orTree(array, n3 + 1, n2)));
                }
                CompletableFuture completableFuture5 = completableFuture4;
                if (completableFuture3 != null) {
                    if (!completableFuture.orRelay(completableFuture2, completableFuture5)) {
                        CompletableFuture.OrRelay o = new CompletableFuture.OrRelay(completableFuture, completableFuture2, completableFuture5);
                        completableFuture2.orpush(completableFuture5, o);
                        o.tryFire(0);
                        return completableFuture;
                    }
                    return completableFuture;
                }
            }
            throw new NullPointerException();
        }
        return completableFuture;
    }
    
    static <U> CompletableFuture<U> asyncSupplyStage(Executor executor, Supplier<U> supplier) {
        if (supplier == null) {
            throw new NullPointerException();
        }
        CompletableFuture<Object> completableFuture = new CompletableFuture();
        executor.execute(new CompletableFuture.AsyncSupply(completableFuture, supplier));
        return completableFuture;
    }
    
    static CompletableFuture<Void> asyncRunStage(Executor executor, Runnable runnable) {
        if (runnable == null) {
            throw new NullPointerException();
        }
        CompletableFuture<Object> completableFuture = new CompletableFuture();
        executor.execute(new CompletableFuture.AsyncRun(completableFuture, runnable));
        return completableFuture;
    }
    
    private Object waitingGet(boolean b) {
        Object o = null;
        boolean tryPushStack = false;
        int spins = -1;
        Object result;
        while ((result = this.result) == null) {
            if (spins < 0) {
                spins = CompletableFuture.SPINS;
            }
            else if (spins > 0) {
                if (ThreadLocalRandom.nextSecondarySeed() < 0) {
                    continue;
                }
                --spins;
            }
            else if (o == null) {
                o = new CompletableFuture.Signaller(b, 0L, 0L);
            }
            else if (tryPushStack == 0) {
                tryPushStack = this.tryPushStack(o);
            }
            else {
                if (b && o.interruptControl < 0) {
                    o.thread = null;
                    this.cleanStack();
                    return null;
                }
                if (o.thread == null || this.result != null) {
                    continue;
                }
                try {
                    ForkJoinPool.managedBlock(o);
                }
                catch (InterruptedException ex) {
                    o.interruptControl = -1;
                }
            }
        }
        if (o != null) {
            o.thread = null;
            if (o.interruptControl < 0) {
                if (b) {
                    result = null;
                }
                else {
                    Thread.currentThread().interrupt();
                }
            }
        }
        this.postComplete();
        return result;
    }
    
    private Object timedGet(long n) throws TimeoutException {
        if (Thread.interrupted()) {
            return null;
        }
        if (n <= 0L) {
            throw new TimeoutException();
        }
        long n2 = System.nanoTime() + n;
        CompletableFuture.Signaller o = new CompletableFuture.Signaller(true, n, (n2 == 0L) ? 1L : n2);
        boolean tryPushStack = false;
        Object result;
        while ((result = this.result) == null) {
            if (tryPushStack == 0) {
                tryPushStack = this.tryPushStack(o);
            }
            else if (o.interruptControl < 0 || o.nanos <= 0L) {
                o.thread = null;
                this.cleanStack();
                if (o.interruptControl < 0) {
                    return null;
                }
                throw new TimeoutException();
            }
            else {
                if (o.thread == null || this.result != null) {
                    continue;
                }
                try {
                    ForkJoinPool.managedBlock(o);
                }
                catch (InterruptedException ex) {
                    o.interruptControl = -1;
                }
            }
        }
        if (o.interruptControl < 0) {
            result = null;
        }
        o.thread = null;
        this.postComplete();
        return result;
    }
    
    public CompletableFuture() {
    }
    
    private CompletableFuture(Object result) {
        this.result = result;
    }
    
    public static <U> CompletableFuture<U> supplyAsync(Supplier<U> supplier) {
        return asyncSupplyStage(CompletableFuture.asyncPool, supplier);
    }
    
    public static <U> CompletableFuture<U> supplyAsync(Supplier<U> supplier, Executor executor) {
        return asyncSupplyStage(screenExecutor(executor), supplier);
    }
    
    public static CompletableFuture<Void> runAsync(Runnable runnable) {
        return asyncRunStage(CompletableFuture.asyncPool, runnable);
    }
    
    public static CompletableFuture<Void> runAsync(Runnable runnable, Executor executor) {
        return asyncRunStage(screenExecutor(executor), runnable);
    }
    
    public static <U> CompletableFuture<U> completedFuture(U u) {
        return new CompletableFuture((u == null) ? CompletableFuture.NIL : u);
    }
    
    @Override
    public boolean isDone() {
        return this.result != null;
    }
    
    @Override
    public T get() throws InterruptedException, ExecutionException {
        Object result;
        return reportGet(((result = this.result) == null) ? this.waitingGet(true) : result);
    }
    
    @Override
    public T get(long n, TimeUnit timeUnit) throws InterruptedException, ExecutionException, TimeoutException {
        long nanos = timeUnit.toNanos(n);
        Object result;
        return reportGet(((result = this.result) == null) ? this.timedGet(nanos) : result);
    }
    
    public T join() {
        Object result;
        return reportJoin(((result = this.result) == null) ? this.waitingGet(false) : result);
    }
    
    public T getNow(T t) {
        Object result;
        return ((result = this.result) == null) ? t : reportJoin(result);
    }
    
    public boolean complete(T t) {
        int completeValue = this.completeValue(t);
        this.postComplete();
        return completeValue;
    }
    
    public boolean completeExceptionally(Throwable t) {
        if (t == null) {
            throw new NullPointerException();
        }
        int internalComplete = this.internalComplete(new AltResult(t));
        this.postComplete();
        return internalComplete;
    }
    
    @Override
    public <U> CompletableFuture<U> thenApply(Function<? super T, ? extends U> function) {
        return this.uniApplyStage(null, function);
    }
    
    @Override
    public <U> CompletableFuture<U> thenApplyAsync(Function<? super T, ? extends U> function) {
        return this.uniApplyStage(CompletableFuture.asyncPool, function);
    }
    
    @Override
    public <U> CompletableFuture<U> thenApplyAsync(Function<? super T, ? extends U> function, Executor executor) {
        return this.uniApplyStage(screenExecutor(executor), function);
    }
    
    @Override
    public CompletableFuture<Void> thenAccept(Consumer<? super T> consumer) {
        return this.uniAcceptStage(null, consumer);
    }
    
    @Override
    public CompletableFuture<Void> thenAcceptAsync(Consumer<? super T> consumer) {
        return this.uniAcceptStage(CompletableFuture.asyncPool, consumer);
    }
    
    @Override
    public CompletableFuture<Void> thenAcceptAsync(Consumer<? super T> consumer, Executor executor) {
        return this.uniAcceptStage(screenExecutor(executor), consumer);
    }
    
    @Override
    public CompletableFuture<Void> thenRun(Runnable runnable) {
        return this.uniRunStage(null, runnable);
    }
    
    @Override
    public CompletableFuture<Void> thenRunAsync(Runnable runnable) {
        return this.uniRunStage(CompletableFuture.asyncPool, runnable);
    }
    
    @Override
    public CompletableFuture<Void> thenRunAsync(Runnable runnable, Executor executor) {
        return this.uniRunStage(screenExecutor(executor), runnable);
    }
    
    @Override
    public <U, V> CompletableFuture<V> thenCombine(CompletionStage<? extends U> completionStage, BiFunction<? super T, ? super U, ? extends V> biFunction) {
        return this.biApplyStage(null, completionStage, biFunction);
    }
    
    @Override
    public <U, V> CompletableFuture<V> thenCombineAsync(CompletionStage<? extends U> completionStage, BiFunction<? super T, ? super U, ? extends V> biFunction) {
        return this.biApplyStage(CompletableFuture.asyncPool, completionStage, biFunction);
    }
    
    @Override
    public <U, V> CompletableFuture<V> thenCombineAsync(CompletionStage<? extends U> completionStage, BiFunction<? super T, ? super U, ? extends V> biFunction, Executor executor) {
        return this.biApplyStage(screenExecutor(executor), completionStage, biFunction);
    }
    
    @Override
    public <U> CompletableFuture<Void> thenAcceptBoth(CompletionStage<? extends U> completionStage, BiConsumer<? super T, ? super U> biConsumer) {
        return this.biAcceptStage(null, completionStage, biConsumer);
    }
    
    @Override
    public <U> CompletableFuture<Void> thenAcceptBothAsync(CompletionStage<? extends U> completionStage, BiConsumer<? super T, ? super U> biConsumer) {
        return this.biAcceptStage(CompletableFuture.asyncPool, completionStage, biConsumer);
    }
    
    @Override
    public <U> CompletableFuture<Void> thenAcceptBothAsync(CompletionStage<? extends U> completionStage, BiConsumer<? super T, ? super U> biConsumer, Executor executor) {
        return this.biAcceptStage(screenExecutor(executor), completionStage, biConsumer);
    }
    
    @Override
    public CompletableFuture<Void> runAfterBoth(CompletionStage<?> completionStage, Runnable runnable) {
        return this.biRunStage(null, completionStage, runnable);
    }
    
    @Override
    public CompletableFuture<Void> runAfterBothAsync(CompletionStage<?> completionStage, Runnable runnable) {
        return this.biRunStage(CompletableFuture.asyncPool, completionStage, runnable);
    }
    
    @Override
    public CompletableFuture<Void> runAfterBothAsync(CompletionStage<?> completionStage, Runnable runnable, Executor executor) {
        return this.biRunStage(screenExecutor(executor), completionStage, runnable);
    }
    
    @Override
    public <U> CompletableFuture<U> applyToEither(CompletionStage<? extends T> completionStage, Function<? super T, U> function) {
        return this.orApplyStage(null, completionStage, function);
    }
    
    @Override
    public <U> CompletableFuture<U> applyToEitherAsync(CompletionStage<? extends T> completionStage, Function<? super T, U> function) {
        return this.orApplyStage(CompletableFuture.asyncPool, completionStage, function);
    }
    
    @Override
    public <U> CompletableFuture<U> applyToEitherAsync(CompletionStage<? extends T> completionStage, Function<? super T, U> function, Executor executor) {
        return this.orApplyStage(screenExecutor(executor), completionStage, function);
    }
    
    @Override
    public CompletableFuture<Void> acceptEither(CompletionStage<? extends T> completionStage, Consumer<? super T> consumer) {
        return this.orAcceptStage(null, completionStage, consumer);
    }
    
    @Override
    public CompletableFuture<Void> acceptEitherAsync(CompletionStage<? extends T> completionStage, Consumer<? super T> consumer) {
        return this.orAcceptStage(CompletableFuture.asyncPool, completionStage, consumer);
    }
    
    @Override
    public CompletableFuture<Void> acceptEitherAsync(CompletionStage<? extends T> completionStage, Consumer<? super T> consumer, Executor executor) {
        return this.orAcceptStage(screenExecutor(executor), completionStage, consumer);
    }
    
    @Override
    public CompletableFuture<Void> runAfterEither(CompletionStage<?> completionStage, Runnable runnable) {
        return this.orRunStage(null, completionStage, runnable);
    }
    
    @Override
    public CompletableFuture<Void> runAfterEitherAsync(CompletionStage<?> completionStage, Runnable runnable) {
        return this.orRunStage(CompletableFuture.asyncPool, completionStage, runnable);
    }
    
    @Override
    public CompletableFuture<Void> runAfterEitherAsync(CompletionStage<?> completionStage, Runnable runnable, Executor executor) {
        return this.orRunStage(screenExecutor(executor), completionStage, runnable);
    }
    
    @Override
    public <U> CompletableFuture<U> thenCompose(Function<? super T, ? extends CompletionStage<U>> function) {
        return this.uniComposeStage(null, function);
    }
    
    @Override
    public <U> CompletableFuture<U> thenComposeAsync(Function<? super T, ? extends CompletionStage<U>> function) {
        return this.uniComposeStage(CompletableFuture.asyncPool, function);
    }
    
    @Override
    public <U> CompletableFuture<U> thenComposeAsync(Function<? super T, ? extends CompletionStage<U>> function, Executor executor) {
        return this.uniComposeStage(screenExecutor(executor), function);
    }
    
    @Override
    public CompletableFuture<T> whenComplete(BiConsumer<? super T, ? super Throwable> biConsumer) {
        return this.uniWhenCompleteStage(null, biConsumer);
    }
    
    @Override
    public CompletableFuture<T> whenCompleteAsync(BiConsumer<? super T, ? super Throwable> biConsumer) {
        return this.uniWhenCompleteStage(CompletableFuture.asyncPool, biConsumer);
    }
    
    @Override
    public CompletableFuture<T> whenCompleteAsync(BiConsumer<? super T, ? super Throwable> biConsumer, Executor executor) {
        return this.uniWhenCompleteStage(screenExecutor(executor), biConsumer);
    }
    
    @Override
    public <U> CompletableFuture<U> handle(BiFunction<? super T, Throwable, ? extends U> biFunction) {
        return this.uniHandleStage(null, biFunction);
    }
    
    @Override
    public <U> CompletableFuture<U> handleAsync(BiFunction<? super T, Throwable, ? extends U> biFunction) {
        return this.uniHandleStage(CompletableFuture.asyncPool, biFunction);
    }
    
    @Override
    public <U> CompletableFuture<U> handleAsync(BiFunction<? super T, Throwable, ? extends U> biFunction, Executor executor) {
        return this.uniHandleStage(screenExecutor(executor), biFunction);
    }
    
    @Override
    public CompletableFuture<T> toCompletableFuture() {
        return this;
    }
    
    @Override
    public CompletableFuture<T> exceptionally(Function<Throwable, ? extends T> function) {
        return this.uniExceptionallyStage(function);
    }
    
    public static CompletableFuture<Void> allOf(CompletableFuture<?>... array) {
        return andTree(array, 0, array.length - 1);
    }
    
    public static CompletableFuture<Object> anyOf(CompletableFuture<?>... array) {
        return orTree(array, 0, array.length - 1);
    }
    
    @Override
    public boolean cancel(boolean b) {
        int n = this.result == null && this.internalComplete(new CompletableFuture.AltResult(new CancellationException()));
        this.postComplete();
        return n || this.isCancelled();
    }
    
    @Override
    public boolean isCancelled() {
        Object result;
        return (result = this.result) instanceof CompletableFuture.AltResult && ((CompletableFuture.AltResult)result).ex instanceof CancellationException;
    }
    
    public boolean isCompletedExceptionally() {
        Object result;
        return (result = this.result) instanceof CompletableFuture.AltResult && result != CompletableFuture.NIL;
    }
    
    public void obtrudeValue(T t) {
        this.result = ((t == null) ? CompletableFuture.NIL : t);
        this.postComplete();
    }
    
    public void obtrudeException(Throwable t) {
        if (t == null) {
            throw new NullPointerException();
        }
        this.result = new AltResult(t);
        this.postComplete();
        return;
    }
    
    public int getNumberOfDependents() {
        int n = 0;
        for (CompletableFuture.Completion completion = this.stack; completion != null; completion = completion.next) {
            ++n;
        }
        return n;
    }
    
    @Override
    public String toString() {
        Object result = this.result;
        int numberOfDependents;
        return super.toString() + ((result == null) ? (((numberOfDependents = this.getNumberOfDependents()) == 0) ? "[Not completed]" : ("[Not completed, " + numberOfDependents + " dependents]")) : ((result instanceof CompletableFuture.AltResult && ((CompletableFuture.AltResult)result).ex != null) ? "[Completed exceptionally]" : "[Completed normally]"));
    }
    
    static {
        NIL = new CompletableFuture.AltResult(null);
        useCommonPool = (ForkJoinPool.getCommonPoolParallelism() > 1);
        asyncPool = (CompletableFuture.useCommonPool ? ForkJoinPool.commonPool() : new CompletableFuture.ThreadPerTaskExecutor());
        SPINS = ((Runtime.getRuntime().availableProcessors() > 1) ? 256 : 0);
        try {
            Unsafe unsafe = UNSAFE = Unsafe.getUnsafe();
            Class<Object> clazz = CompletableFuture.class;
            RESULT = unsafe.objectFieldOffset(clazz.getDeclaredField("result"));
            STACK = unsafe.objectFieldOffset(clazz.getDeclaredField("stack"));
            NEXT = unsafe.objectFieldOffset(CompletableFuture.Completion.class.getDeclaredField("next"));
        }
        catch (Exception cause) {
            throw new Error(cause);
        }
    }
    
    static final class AltResult
    {
        final Throwable ex;
        
        AltResult(Throwable ex) {
            this.ex = ex;
        }
    }
    
    static final class ThreadPerTaskExecutor implements Executor
    {
        @Override
        public void execute(Runnable task) {
            new Thread(task).start();
        }
    }
    
    abstract static class Completion extends ForkJoinTask<Void> implements Runnable, CompletableFuture.AsynchronousCompletionTask
    {
        volatile CompletableFuture.Completion next;
        
        abstract CompletableFuture<?> tryFire(int p0);
        
        abstract boolean isLive();
        
        @Override
        public final void run() {
            this.tryFire(1);
        }
        
        public final boolean exec() {
            this.tryFire(1);
            return true;
        }
        
        @Override
        public final Void getRawResult() {
            return null;
        }
        
        public final void setRawResult(Void void1) {
        }
    }
    
    abstract static class UniCompletion<T, V> extends CompletableFuture.Completion
    {
        Executor executor;
        CompletableFuture<V> dep;
        CompletableFuture<T> src;
        
        UniCompletion(Executor executor, CompletableFuture<V> dep, CompletableFuture<T> src) {
            this.executor = executor;
            this.dep = dep;
            this.src = src;
        }
        
        final boolean claim() {
            Executor executor = this.executor;
            if (this.compareAndSetForkJoinTaskTag((short)0, (short)1)) {
                if (executor == null) {
                    return true;
                }
                this.executor = null;
                executor.execute(this);
            }
            return false;
        }
        
        @Override
        final boolean isLive() {
            return this.dep != null;
        }
    }
    
    static final class UniApply<T, V> extends CompletableFuture.UniCompletion<T, V>
    {
        Function<? super T, ? extends V> fn;
        
        UniApply(Executor executor, CompletableFuture<V> completableFuture, CompletableFuture<T> completableFuture2, Function<? super T, ? extends V> fn) {
            super(executor, completableFuture, completableFuture2);
            this.fn = fn;
        }
        
        @Override
        final CompletableFuture<V> tryFire(int n) {
            CompletableFuture<Object> dep;
            CompletableFuture<Object> src;
            if ((dep = this.dep) == null || !dep.uniApply(src = this.src, this.fn, (n > 0) ? null : this)) {
                return null;
            }
            this.dep = null;
            this.src = null;
            this.fn = null;
            return dep.postFire(src, n);
        }
    }
    
    static final class UniAccept<T> extends CompletableFuture.UniCompletion<T, Void>
    {
        Consumer<? super T> fn;
        
        UniAccept(Executor executor, CompletableFuture<Void> completableFuture, CompletableFuture<T> completableFuture2, Consumer<? super T> fn) {
            super(executor, completableFuture, completableFuture2);
            this.fn = fn;
        }
        
        @Override
        final CompletableFuture<Void> tryFire(int n) {
            CompletableFuture<Object> dep;
            CompletableFuture<Object> src;
            if ((dep = this.dep) == null || !dep.uniAccept(src = this.src, this.fn, (n > 0) ? null : this)) {
                return null;
            }
            this.dep = null;
            this.src = null;
            this.fn = null;
            return dep.postFire(src, n);
        }
    }
    
    static final class UniRun<T> extends CompletableFuture.UniCompletion<T, Void>
    {
        Runnable fn;
        
        UniRun(Executor executor, CompletableFuture<Void> completableFuture, CompletableFuture<T> completableFuture2, Runnable fn) {
            super(executor, completableFuture, completableFuture2);
            this.fn = fn;
        }
        
        @Override
        final CompletableFuture<Void> tryFire(int n) {
            CompletableFuture<Object> dep;
            CompletableFuture<Object> src;
            if ((dep = this.dep) == null || !dep.uniRun(src = this.src, this.fn, (n > 0) ? null : this)) {
                return null;
            }
            this.dep = null;
            this.src = null;
            this.fn = null;
            return dep.postFire(src, n);
        }
    }
    
    static final class UniWhenComplete<T> extends CompletableFuture.UniCompletion<T, T>
    {
        BiConsumer<? super T, ? super Throwable> fn;
        
        UniWhenComplete(Executor executor, CompletableFuture<T> completableFuture, CompletableFuture<T> completableFuture2, BiConsumer<? super T, ? super Throwable> fn) {
            super(executor, completableFuture, completableFuture2);
            this.fn = fn;
        }
        
        @Override
        final CompletableFuture<T> tryFire(int n) {
            CompletableFuture<Object> dep;
            CompletableFuture<Object> src;
            if ((dep = this.dep) == null || !dep.uniWhenComplete(src = this.src, this.fn, (n > 0) ? null : this)) {
                return null;
            }
            this.dep = null;
            this.src = null;
            this.fn = null;
            return dep.postFire(src, n);
        }
    }
    
    static final class UniHandle<T, V> extends CompletableFuture.UniCompletion<T, V>
    {
        BiFunction<? super T, Throwable, ? extends V> fn;
        
        UniHandle(Executor executor, CompletableFuture<V> completableFuture, CompletableFuture<T> completableFuture2, BiFunction<? super T, Throwable, ? extends V> fn) {
            super(executor, completableFuture, completableFuture2);
            this.fn = fn;
        }
        
        @Override
        final CompletableFuture<V> tryFire(int n) {
            CompletableFuture<Object> dep;
            CompletableFuture<Object> src;
            if ((dep = this.dep) == null || !dep.uniHandle(src = this.src, this.fn, (n > 0) ? null : this)) {
                return null;
            }
            this.dep = null;
            this.src = null;
            this.fn = null;
            return dep.postFire(src, n);
        }
    }
    
    static final class UniExceptionally<T> extends CompletableFuture.UniCompletion<T, T>
    {
        Function<? super Throwable, ? extends T> fn;
        
        UniExceptionally(CompletableFuture<T> completableFuture, CompletableFuture<T> completableFuture2, Function<? super Throwable, ? extends T> fn) {
            super(null, completableFuture, completableFuture2);
            this.fn = fn;
        }
        
        @Override
        final CompletableFuture<T> tryFire(int n) {
            CompletableFuture<Object> dep;
            CompletableFuture<Object> src;
            if ((dep = this.dep) == null || !dep.uniExceptionally(src = this.src, this.fn, this)) {
                return null;
            }
            this.dep = null;
            this.src = null;
            this.fn = null;
            return dep.postFire(src, n);
        }
    }
    
    static final class UniRelay<T> extends CompletableFuture.UniCompletion<T, T>
    {
        UniRelay(CompletableFuture<T> completableFuture, CompletableFuture<T> completableFuture2) {
            super(null, completableFuture, completableFuture2);
        }
        
        @Override
        final CompletableFuture<T> tryFire(int n) {
            CompletableFuture<Object> dep;
            CompletableFuture<Object> src;
            if ((dep = this.dep) == null || !dep.uniRelay(src = this.src)) {
                return null;
            }
            this.src = null;
            this.dep = null;
            return dep.postFire(src, n);
        }
    }
    
    static final class UniCompose<T, V> extends CompletableFuture.UniCompletion<T, V>
    {
        Function<? super T, ? extends CompletionStage<V>> fn;
        
        UniCompose(Executor executor, CompletableFuture<V> completableFuture, CompletableFuture<T> completableFuture2, Function<? super T, ? extends CompletionStage<V>> fn) {
            super(executor, completableFuture, completableFuture2);
            this.fn = fn;
        }
        
        @Override
        final CompletableFuture<V> tryFire(int n) {
            CompletableFuture<Object> dep;
            CompletableFuture<Object> src;
            if ((dep = this.dep) == null || !dep.uniCompose(src = this.src, this.fn, (n > 0) ? null : this)) {
                return null;
            }
            this.dep = null;
            this.src = null;
            this.fn = null;
            return dep.postFire(src, n);
        }
    }
    
    abstract static class BiCompletion<T, U, V> extends CompletableFuture.UniCompletion<T, V>
    {
        CompletableFuture<U> snd;
        
        BiCompletion(Executor executor, CompletableFuture<V> completableFuture, CompletableFuture<T> completableFuture2, CompletableFuture<U> snd) {
            super(executor, completableFuture, completableFuture2);
            this.snd = snd;
        }
    }
    
    static final class CoCompletion extends CompletableFuture.Completion
    {
        CompletableFuture.BiCompletion<?, ?, ?> base;
        
        CoCompletion(CompletableFuture.BiCompletion<?, ?, ?> base) {
            this.base = base;
        }
        
        @Override
        final CompletableFuture<?> tryFire(int n) {
            CompletableFuture.BiCompletion<Object, Object, Object> base;
            CompletableFuture<Object> tryFire;
            if ((base = this.base) == null || (tryFire = base.tryFire(n)) == null) {
                return null;
            }
            this.base = null;
            return tryFire;
        }
        
        @Override
        final boolean isLive() {
            CompletableFuture.BiCompletion<Object, Object, Object> base;
            return (base = this.base) != null && base.dep != null;
        }
    }
    
    static final class BiApply<T, U, V> extends CompletableFuture.BiCompletion<T, U, V>
    {
        BiFunction<? super T, ? super U, ? extends V> fn;
        
        BiApply(Executor executor, CompletableFuture<V> completableFuture, CompletableFuture<T> completableFuture2, CompletableFuture<U> completableFuture3, BiFunction<? super T, ? super U, ? extends V> fn) {
            super(executor, completableFuture, completableFuture2, completableFuture3);
            this.fn = fn;
        }
        
        @Override
        final CompletableFuture<V> tryFire(int n) {
            CompletableFuture<Object> dep;
            if ((dep = this.dep) != null) {
                Future<T> future = dep;
                CompletableFuture<T> src = this.src;
                CompletableFuture<Object> snd;
                if (future.biApply(src, snd = this.snd, this.fn, (n > 0) ? null : this)) {
                    this.dep = null;
                    this.src = null;
                    this.snd = null;
                    this.fn = null;
                    return dep.postFire(src, snd, n);
                }
            }
            return null;
        }
    }
    
    static final class BiAccept<T, U> extends CompletableFuture.BiCompletion<T, U, Void>
    {
        BiConsumer<? super T, ? super U> fn;
        
        BiAccept(Executor executor, CompletableFuture<Void> completableFuture, CompletableFuture<T> completableFuture2, CompletableFuture<U> completableFuture3, BiConsumer<? super T, ? super U> fn) {
            super(executor, completableFuture, completableFuture2, completableFuture3);
            this.fn = fn;
        }
        
        @Override
        final CompletableFuture<Void> tryFire(int n) {
            CompletableFuture<Object> dep;
            if ((dep = this.dep) != null) {
                CompletableFuture<U> completableFuture = dep;
                CompletableFuture<T> src = this.src;
                CompletableFuture<Object> snd;
                if (completableFuture.biAccept(src, snd = this.snd, this.fn, (n > 0) ? null : this)) {
                    this.dep = null;
                    this.src = null;
                    this.snd = null;
                    this.fn = null;
                    return dep.postFire(src, snd, n);
                }
            }
            return null;
        }
    }
    
    static final class BiRun<T, U> extends CompletableFuture.BiCompletion<T, U, Void>
    {
        Runnable fn;
        
        BiRun(Executor executor, CompletableFuture<Void> completableFuture, CompletableFuture<T> completableFuture2, CompletableFuture<U> completableFuture3, Runnable fn) {
            super(executor, completableFuture, completableFuture2, completableFuture3);
            this.fn = fn;
        }
        
        @Override
        final CompletableFuture<Void> tryFire(int n) {
            CompletableFuture<Object> dep;
            if ((dep = this.dep) != null) {
                CompletableFuture<U> completableFuture = dep;
                CompletableFuture<Object> src = this.src;
                CompletableFuture<Object> snd;
                if (completableFuture.biRun(src, snd = this.snd, this.fn, (n > 0) ? null : this)) {
                    this.dep = null;
                    this.src = null;
                    this.snd = null;
                    this.fn = null;
                    return dep.postFire(src, snd, n);
                }
            }
            return null;
        }
    }
    
    static final class BiRelay<T, U> extends CompletableFuture.BiCompletion<T, U, Void>
    {
        BiRelay(CompletableFuture<Void> completableFuture, CompletableFuture<T> completableFuture2, CompletableFuture<U> completableFuture3) {
            super(null, completableFuture, completableFuture2, completableFuture3);
        }
        
        @Override
        final CompletableFuture<Void> tryFire(int n) {
            CompletableFuture<Object> dep;
            if ((dep = this.dep) != null) {
                CompletableFuture<U> completableFuture = dep;
                CompletableFuture<Object> src = this.src;
                CompletableFuture<Object> snd;
                if (completableFuture.biRelay(src, snd = this.snd)) {
                    this.src = null;
                    this.snd = null;
                    this.dep = null;
                    return dep.postFire(src, snd, n);
                }
            }
            return null;
        }
    }
    
    static final class OrApply<T, U extends T, V> extends CompletableFuture.BiCompletion<T, U, V>
    {
        Function<? super T, ? extends V> fn;
        
        OrApply(Executor executor, CompletableFuture<V> completableFuture, CompletableFuture<T> completableFuture2, CompletableFuture<U> completableFuture3, Function<? super T, ? extends V> fn) {
            super(executor, completableFuture, completableFuture2, completableFuture3);
            this.fn = fn;
        }
        
        @Override
        final CompletableFuture<V> tryFire(int n) {
            CompletableFuture<Object> dep;
            if ((dep = this.dep) != null) {
                Future<T> future = dep;
                CompletableFuture<T> src = this.src;
                CompletableFuture<Object> snd;
                if (future.orApply(src, snd = this.snd, this.fn, (n > 0) ? null : this)) {
                    this.dep = null;
                    this.src = null;
                    this.snd = null;
                    this.fn = null;
                    return dep.postFire(src, snd, n);
                }
            }
            return null;
        }
    }
    
    static final class OrAccept<T, U extends T> extends CompletableFuture.BiCompletion<T, U, Void>
    {
        Consumer<? super T> fn;
        
        OrAccept(Executor executor, CompletableFuture<Void> completableFuture, CompletableFuture<T> completableFuture2, CompletableFuture<U> completableFuture3, Consumer<? super T> fn) {
            super(executor, completableFuture, completableFuture2, completableFuture3);
            this.fn = fn;
        }
        
        @Override
        final CompletableFuture<Void> tryFire(int n) {
            CompletableFuture<Object> dep;
            if ((dep = this.dep) != null) {
                CompletableFuture<U> completableFuture = dep;
                CompletableFuture<T> src = this.src;
                CompletableFuture<Object> snd;
                if (completableFuture.orAccept(src, snd = this.snd, this.fn, (n > 0) ? null : this)) {
                    this.dep = null;
                    this.src = null;
                    this.snd = null;
                    this.fn = null;
                    return dep.postFire(src, snd, n);
                }
            }
            return null;
        }
    }
    
    static final class OrRun<T, U> extends CompletableFuture.BiCompletion<T, U, Void>
    {
        Runnable fn;
        
        OrRun(Executor executor, CompletableFuture<Void> completableFuture, CompletableFuture<T> completableFuture2, CompletableFuture<U> completableFuture3, Runnable fn) {
            super(executor, completableFuture, completableFuture2, completableFuture3);
            this.fn = fn;
        }
        
        @Override
        final CompletableFuture<Void> tryFire(int n) {
            CompletableFuture<Object> dep;
            if ((dep = this.dep) != null) {
                CompletableFuture<U> completableFuture = dep;
                CompletableFuture<Object> src = this.src;
                CompletableFuture<Object> snd;
                if (completableFuture.orRun(src, snd = this.snd, this.fn, (n > 0) ? null : this)) {
                    this.dep = null;
                    this.src = null;
                    this.snd = null;
                    this.fn = null;
                    return dep.postFire(src, snd, n);
                }
            }
            return null;
        }
    }
    
    static final class OrRelay<T, U> extends CompletableFuture.BiCompletion<T, U, Object>
    {
        OrRelay(CompletableFuture<Object> completableFuture, CompletableFuture<T> completableFuture2, CompletableFuture<U> completableFuture3) {
            super(null, completableFuture, completableFuture2, completableFuture3);
        }
        
        @Override
        final CompletableFuture<Object> tryFire(int n) {
            CompletableFuture<Object> dep;
            if ((dep = this.dep) != null) {
                CompletableFuture<U> completableFuture = dep;
                CompletableFuture<Object> src = this.src;
                CompletableFuture<Object> snd;
                if (completableFuture.orRelay(src, snd = this.snd)) {
                    this.src = null;
                    this.snd = null;
                    this.dep = null;
                    return dep.postFire(src, snd, n);
                }
            }
            return null;
        }
    }
    
    static final class AsyncSupply<T> extends ForkJoinTask<Void> implements Runnable, CompletableFuture.AsynchronousCompletionTask
    {
        CompletableFuture<T> dep;
        Supplier<T> fn;
        
        AsyncSupply(CompletableFuture<T> dep, Supplier<T> fn) {
            this.dep = dep;
            this.fn = fn;
        }
        
        @Override
        public final Void getRawResult() {
            return null;
        }
        
        public final void setRawResult(Void void1) {
        }
        
        public final boolean exec() {
            this.run();
            return true;
        }
        
        @Override
        public void run() {
            CompletableFuture<Object> dep;
            Supplier<Object> fn;
            if ((dep = this.dep) != null && (fn = this.fn) != null) {
                this.dep = null;
                this.fn = null;
                if (dep.result == null) {
                    try {
                        dep.completeValue(fn.get());
                    }
                    catch (Throwable t) {
                        dep.completeThrowable(t);
                    }
                }
                dep.postComplete();
            }
        }
    }
    
    static final class AsyncRun extends ForkJoinTask<Void> implements Runnable, CompletableFuture.AsynchronousCompletionTask
    {
        CompletableFuture<Void> dep;
        Runnable fn;
        
        AsyncRun(CompletableFuture<Void> dep, Runnable fn) {
            this.dep = dep;
            this.fn = fn;
        }
        
        @Override
        public final Void getRawResult() {
            return null;
        }
        
        public final void setRawResult(Void void1) {
        }
        
        public final boolean exec() {
            this.run();
            return true;
        }
        
        @Override
        public void run() {
            CompletableFuture<Object> dep;
            Runnable fn;
            if ((dep = this.dep) != null && (fn = this.fn) != null) {
                this.dep = null;
                this.fn = null;
                if (dep.result == null) {
                    try {
                        fn.run();
                        dep.completeNull();
                    }
                    catch (Throwable t) {
                        dep.completeThrowable(t);
                    }
                }
                dep.postComplete();
            }
        }
    }
    
    static final class Signaller extends CompletableFuture.Completion implements ForkJoinPool.ManagedBlocker
    {
        long nanos;
        final long deadline;
        volatile int interruptControl;
        volatile Thread thread;
        
        Signaller(boolean interruptControl, long nanos, long deadline) {
            this.thread = Thread.currentThread();
            this.interruptControl = interruptControl;
            this.nanos = nanos;
            this.deadline = deadline;
        }
        
        @Override
        final CompletableFuture<?> tryFire(int n) {
            Thread thread;
            if ((thread = this.thread) != null) {
                this.thread = null;
                LockSupport.unpark(thread);
            }
            return null;
        }
        
        @Override
        public boolean isReleasable() {
            if (this.thread == null) {
                return true;
            }
            if (Thread.interrupted()) {
                int interruptControl = this.interruptControl;
                this.interruptControl = -1;
                if (interruptControl > 0) {
                    return true;
                }
            }
            if (this.deadline != 0L) {
                if (this.nanos > 0L) {
                    long nanos = this.deadline - System.nanoTime();
                    this.nanos = nanos;
                    if (nanos > 0L) {
                        return false;
                    }
                }
                this.thread = null;
                return true;
            }
            return false;
        }
        
        @Override
        public boolean block() {
            if (this.isReleasable()) {
                return true;
            }
            if (this.deadline == 0L) {
                LockSupport.park(this);
            }
            else if (this.nanos > 0L) {
                LockSupport.parkNanos(this, this.nanos);
            }
            return this.isReleasable();
        }
        
        @Override
        final boolean isLive() {
            return this.thread != null;
        }
    }
    
    public interface AsynchronousCompletionTask
    {
    }
}
